#![feature(slice_split_once)]

use std::collections::HashMap;
use std::fs::File;
use std::io::{BufRead, BufReader};
use std::time::Instant;

use onebrc::{hashstr, parse_temp, StationMeasurements};

const DEFAULT_HASHMAP_LENGTH: usize = 10000;

fn main() {
    let now = Instant::now();
    let mut stations: HashMap<usize, (String, StationMeasurements)> =
        HashMap::with_capacity(DEFAULT_HASHMAP_LENGTH);

    let file = File::open("../../../measurements.txt").expect("File measurements.txt not found");
    let mut reader = BufReader::new(&file);
    let mut line = Vec::with_capacity(108);
    loop {
        let line_len = reader.read_until(b'\n', &mut line).expect("could not read bytes");
        if line_len == 0 {
            break;
        }
        let (station, temp) = line.rsplit_once(|&byte| byte == b';').unwrap();
        let hash = hashstr(station);
        let station = unsafe { String::from_utf8_unchecked(Vec::from(station)) };
        let temp = parse_temp(temp.split_last().unwrap().1);
        let measurements_option = stations.get_mut(&hash);
        if let Some((_, measurements)) = measurements_option {
            measurements.update(temp);
        } else {
            let measurements = StationMeasurements {
                min: temp,
                max: temp,
                count: 1,
                sum: temp,
            };
            stations.insert(hash, (station, measurements));
        }
        line.clear();
    }
    let mut stations: Vec<String> = stations.iter().map(|(_ , (station, measurements))| {
        let measurements = measurements.to_string();
        format!("{station}={measurements}")
    }).collect();
    stations.sort();
    let stations = stations.join(",");
    println!("{{{stations}}}");
    println!("Time={} μs", now.elapsed().as_micros());
}

